@@ -116,10 +116,13 @@ gt :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
 gt = (.>)
 
 (.<=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-(.<=) = BinaryOp "leq" (<=)
+(.<=) (Lit l) (Lit r) = Lit (l <= r)
+(.<=) (Lit l) expr = UnaryOp ("leq " <> T.pack (show l)) (l <=) expr
+(.<=) expr (Lit r) = UnaryOp ("gt " <> T.pack (show r)) (r >) expr
+(.<=) l r = BinaryOp "leq" (<=) l r
 
 leq :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
-leq = BinaryOp "leq" (<=)
+leq = (.<=)
 
 (.>=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
 (.>=) = BinaryOp "geq" (>=)
@@ -166,7 +169,7 @@ maximum :: (Columnable a, Ord a) => Expr a -> Expr a
 maximum expr = AggReduce expr "maximum" Prelude.max
 
 sum :: forall a. (Columnable a, Num a) => Expr a -> Expr a
-sum expr = AggFold expr "sum" 0 (+)
+sum expr = AggReduce expr "sum" (+)
 
 sumMaybe :: forall a. (Columnable a, Num a) => Expr (Maybe a) -> Expr a
 sumMaybe expr = AggVector expr "sumMaybe" (P.sum . catMaybes . V.toList)
