@@ -280,7 +280,7 @@ mapColumn f = \case
         | Just Refl <- testEquality (typeRep @a) (typeRep @b) ->
             Right $ case sUnbox @c of
                 STrue -> UnboxedColumn (VU.map f col)
-                SFalse -> fromVector @c (VB.map f (VB.convert col))
+                SFalse -> fromVector @c (VB.generate (VU.length col) (f . VU.unsafeIndex col))
         | otherwise ->
             Left $
                 TypeMismatchException
@@ -291,6 +291,10 @@ mapColumn f = \case
                         , errorColumnName = Nothing
                         }
                     )
+{-# SPECIALIZE mapColumn ::
+    (Double -> Double) -> Column -> Either DataFrameException Column
+    #-}
+{-# INLINEABLE mapColumn #-}
 
 -- | O(1) Gets the number of elements in the column.
 columnLength :: Column -> Int
@@ -606,6 +610,88 @@ ifoldlColumn f acc c@(UnboxedColumn (column :: VU.Vector d)) = case testEquality
                     }
                 )
 
+foldlColumn ::
+    forall a b.
+    (Columnable a, Columnable b) =>
+    (b -> a -> b) -> b -> Column -> Either DataFrameException b
+foldlColumn f acc c@(BoxedColumn (column :: VB.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl' f acc column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldlColumn"
+                    , errorColumnName = Nothing
+                    }
+                )
+foldlColumn f acc c@(OptionalColumn (column :: VB.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl' f acc column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldlColumn"
+                    , errorColumnName = Nothing
+                    }
+                )
+foldlColumn f acc c@(UnboxedColumn (column :: VU.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl' f acc column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldlColumn"
+                    , errorColumnName = Nothing
+                    }
+                )
+
+foldl1Column ::
+    forall a.
+    (Columnable a) =>
+    (a -> a -> a) -> Column -> Either DataFrameException a
+foldl1Column f c@(BoxedColumn (column :: VB.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl1' f column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldl1Column"
+                    , errorColumnName = Nothing
+                    }
+                )
+foldl1Column f c@(OptionalColumn (column :: VB.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl1' f column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldl1Column"
+                    , errorColumnName = Nothing
+                    }
+                )
+foldl1Column f c@(UnboxedColumn (column :: VU.Vector d)) = case testEquality (typeRep @a) (typeRep @d) of
+    Just Refl -> pure $ VG.foldl1' f column
+    Nothing ->
+        Left $
+            TypeMismatchException
+                ( MkTypeErrorContext
+                    { userType = Right (typeRep @a)
+                    , expectedType = Right (typeRep @d)
+                    , callingFunctionName = Just "foldl1Column"
+                    , errorColumnName = Nothing
+                    }
+                )
+
 headColumn :: forall a. (Columnable a) => Column -> Either DataFrameException a
 headColumn (BoxedColumn (col :: VB.Vector b)) = case testEquality (typeRep @a) (typeRep @b) of
     Just Refl ->
@@ -803,7 +889,7 @@ freezeColumn' nulls (MBoxedColumn col)
             . VB.imap
                 ( \i v ->
                     if i `elem` map fst nulls
-                        then Left (fromMaybe (error "") (lookup i nulls))
+                        then Left (fromMaybe (error "UNEXPECTED ERROR DURING FREEZE") (lookup i nulls))
                         else Right v
                 )
             <$> VB.unsafeFreeze col
@@ -824,7 +910,7 @@ freezeColumn' nulls (MUnboxedColumn col)
                         (VU.length c)
                         ( \i ->
                             if i `elem` map fst nulls
-                                then Left (fromMaybe (error "") (lookup i nulls))
+                                then Left (fromMaybe (error "UNEXPECTED ERROR DURING FREEZE") (lookup i nulls))
                                 else Right (c VU.! i)
                         )
 {-# INLINE freezeColumn' #-}
@@ -1265,7 +1351,7 @@ toUnboxedVector column =
                         ( MkTypeErrorContext
                             { userType = Right (typeRep @Int)
                             , expectedType = Right (typeRep @a)
-                            , callingFunctionName = Just "toIntVector"
+                            , callingFunctionName = Just "toUnboxedVector"
                             , errorColumnName = Nothing
                             }
                         )
@@ -1279,3 +1365,7 @@ toUnboxedVector column =
                         , errorColumnName = Nothing
                         }
                     )
+{-# SPECIALIZE toUnboxedVector ::
+    Column -> Either DataFrameException (VU.Vector Double)
+    #-}
+{-# INLINE toUnboxedVector #-}
